from shaderdef.glsl_var import GlslVar
from shaderdef.interface import (AttributeBlock,
                                 FragmentShaderOutputBlock, GlGsIn,
                                 ShaderInterface, UniformBlock)
from shaderdef.shader import ShaderDef
