# pylint: disable=unused-argument

def geom_shader_meta(input_primitive, output_primitive, max_vertices):
    def passthrough(func):
        return func
    return passthrough

def end_primitive():
    pass

def exp2(var):
    pass

def mod(num1, num2):
    pass

def length(vec_type):
    pass
